// CheapMocap.cpp: dfinit le point d'entre pour l'application console.

// Include files
#include "stdafx.h"
#include "cv.h"
#include "highgui.h"
#include "../Common/Scene.h"
#include "../Common/Camera.h"
#include <vector>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <conio.h>
#include "Clients.h"
#include <sys/types.h>
#include <winsock2.h>
#include <cstdio>
#include <time.h>
#include <conio.h>
#include <QtCore>
#include <iostream>

extern int VERBOSE_MODE;

// Libraries
#pragma comment(lib,"ws2_32.lib")
#define BUF_SIZE 65535
// Definitions
WSADATA initialisation_win32; // Initialisation variable
int error; // Error variable
SOCKET sdServer,listenfd; // Sockets IDs
SOCKADDR_IN clientInfos; // Informations bound to client
SOCKADDR_IN servAddr; // Informations bound to server

fd_set readfs; // Read
fd_set writefs; // Write

unsigned int maxdescriptor=0; // Maxdescriptor initialisation
SOCKET connfd[255]; // Socket array
int clientAlive[255]; // Client alive notificiation Array
int captureReceived[255]; // Capture received notification Array
bool captureOK=true;

char* data=(char*)malloc(BUF_SIZE*sizeof(char));

int clientMax=-1; // Id of the client whose id is the greatest
int nbClient=0; // Nb of clients connected

// Set a buffer to 0
void zero(char * tab,int taille)
{
	for (int i=0;i<taille;i++)
	{
		tab[i]=0;
	}
}


int initNetwork(); // Initialise the network


int main (int argc, char* argv[])
{
	// Verbose mode handling
	if (argc>1){
		VERBOSE_MODE=1;
		cout << "Verbose mode ON" << endl;
	}else{
		VERBOSE_MODE=0;	
		cout << "Verbose mode OFF" << endl;
	}

	int phase=0; // Phase initialisation

	std::cout << "Calibr Server !" << endl;
	std::cout << "-----------------------------------------------------------" << endl << endl;
	std::cout << "Usage :" << endl;
	std::cout << "[i] : send camera initialisation request to clients" << endl;
	std::cout << "[c] : send single capture shot request to clients" << endl;
	std::cout << endl << "Network Initializing ..." << endl;

	initNetwork(); // Network initialisation

	std::cout << "Initialization complete ... Running ..." << endl << endl;
	

	struct timeval tv; // Timeout value


	memset(data,0,sizeof(data));
	int data_len = sizeof(data); // Length initialisation

	int client_len; // Length

	bool running = true;

	// Main loop
	while(running)
	{
	
		FD_ZERO(&readfs); // Read fd_set set to 0
		FD_SET(listenfd,&readfs); // Listening socket set in read fd_set
	
		// For each alive client, we monitor its socket
		for(int i=0;i<=clientMax;i++)
		{
			if(clientAlive[i]==0)
				continue;
			FD_SET(connfd[i],&readfs);
		}

		// Timeout reinitialisation
		tv.tv_sec = 0;
		tv.tv_usec = 500000;

		/* ------------ STATE MACHINE ---------- */
		// If keyboard activity
		if(_kbhit())
		{
			char c=_getch(); // We retrieve the key
		
			if(c==27)
			{
				running=false;
				break;
			}

			// Phase 0 : hitting i starts phase 1
			if(phase==0 && c=='i')
			{
				
				data_len=2;
				phase = 1; // Next phase
	
				// For each alive client, we send an init cam request
				for(int i=0;i<=clientMax;i++)
				{
					if(clientAlive[i]==0)
						continue;

					std::cout << "client " << i << " : init cam request sent" << endl;

					// According to the defined protocol, data in phase 0 must begin with 0x01
					data[0]=0x01;
					data[1]=0x00;

					// We add the client socket in write fd_set, to send him the data
					FD_SET(connfd[i],&writefs);
				}
			}

			// Phase 1 : hitting c ask clients for taking a shot (for calibration)
			if(phase==1 && c=='c' && captureOK)
			{
				captureOK=false;

				for(int j=0;j<=clientMax;j++)
				{
					if(clientAlive[j]==0)
						continue;

					captureReceived[j]=false;
				}

				// For each alive client, we send a shot request
				for(int i=0;i<=clientMax;i++)
				{
					if(clientAlive[i]==0)
						continue;

					std::cout << "client " << i << " : shot request sent" << endl;

					
					data[0]=0x00;
					data[1]=0x00;

					// We add the client socket in write fd_set, to send him the data
					FD_SET(connfd[i],&writefs);
				}
			}
		}


		/* ------------ STATE MACHINE END ---------- */

		// We monitor the activity of the sockets set in the Read and Write fd_set
		int retval = select(maxdescriptor + 1, &readfs, &writefs, NULL , &tv);

		// If 0 : no activity
		if (retval==0)
		{
			//printf("No activity .... clients : %d , phase : %d\n",nbClient,phase);

		}
		else
		{
			// Activity on the listening socket?
			if (FD_ISSET (listenfd, &readfs))
			{
				cout << "Incoming connection!" << endl;

				int newId;

				for(int j=0;j<255;j++)
				{
					if (clientAlive[j]==0)
					{
						newId=j;
						break;
					}
				}

				// We accept a new connection
				connfd[newId] = accept(listenfd, (struct sockaddr *) &clientInfos, &client_len);

				// New client is alive
				clientAlive[newId]=1;

				// We are going to send his id to him
				//sprintf_s(data,,"%d",clientMax);
				sprintf_s(data,BUF_SIZE,"%d",newId);
				

				data_len=5;

				// We add the client socket in write fd_set, to send him the data
				FD_SET(connfd[newId],&writefs);

				for(int j=0;j<255;j++)
				{
					if (clientAlive[j]==1)
					{	
						// We update the maxdescriptor
						if(connfd[j]>maxdescriptor)
							maxdescriptor=connfd[j];
					}

					if (clientAlive[j]==1)
						clientMax=j;

				}

				// We update the counter variables
				nbClient++;

			}

			// Activity on an alive client socket?
			for(int i=0;i<=clientMax;i++)
			{
				if(clientAlive[i]==0)
					continue;

				// Writing activity ?
				if (FD_ISSET (connfd[i], &writefs))
				{
					int r=-1;
					// We send data to the client
					while(r<0)
					{
					r=send(connfd[i], data, data_len, 0);
			
					std::cout << r << " bytes sent to socket " << connfd[i] << " / client " << i << endl;

					Sleep(1);
					}

					// We erase the socket from the writing fd_set
					FD_CLR(connfd[i],&writefs);

					continue;
				}
				
				// Reading activity ?
				if (FD_ISSET (connfd[i], &readfs))
				{
					// We receive data
					data_len=recv(connfd[i], data,BUF_SIZE, 0);

					std::cout << data_len << " bytes received from " << connfd[i] << endl;
					
					// If length < 0 it means that the client has gone offline
					if(data_len<0)
					{
						std::cout << "Client " << i <<" disconnected" << endl;

						// We update the array
						clientAlive[i]=0;

						// We erase its socket from all the fd_sets
						FD_CLR(connfd[i],&readfs);
						FD_CLR(connfd[i],&writefs);

						// We close the connection
						shutdown(connfd[i],2);

						// We update counter
						nbClient--;
						continue;
					}
					else
					{
						std::cout << "Client " << i << " has taken a capture" << endl;
						captureReceived[i]=true;
						captureOK=1;

						for(int j=0;j<=clientMax && captureOK;j++)
						{
							if(clientAlive[j]==0)
								continue;

							captureOK=captureOK&&captureReceived[j];
						}


					}


				}

			}
			
		

		}

		// We reset the Write fd_set
		FD_ZERO(&writefs);

	}


}

int initNetwork(){


	// We set our arrays to 0
	for(int i=0;i<255;i++)
	{
		clientAlive[i]=0;
		captureReceived[i]=0;
	}

	// Initialisation winsock
	error=WSAStartup(MAKEWORD(2,2),&initialisation_win32);
	if (error!=0)
		std::cout << endl << "Winsock initialisation error : "<< error <<WSAGetLastError();
	else
		std::cout << endl << "WSAStartup...OK";

	// Socket initialisation
	listenfd=socket(AF_INET,SOCK_STREAM,0);
	if (listenfd==INVALID_SOCKET)
		std::cout << endl << "Socket creation error : " << WSAGetLastError();
	else
		std::cout << endl << "Socket.......OK";

	// We update the maxdescriptor
	if(listenfd>maxdescriptor)
		maxdescriptor=listenfd;

	// Server informations
	servAddr.sin_family=AF_INET;
	servAddr.sin_addr.s_addr=INADDR_ANY; // Ecoute sur toutes les IP locales 
	servAddr.sin_port=htons(1219); // Ecoute sur le port 1219

	// We bind the socket
	if (bind(listenfd, (struct sockaddr *) &servAddr, sizeof(servAddr)) < 0)
	{
		perror("Bind error\n");
		exit(1); 
	}
	else
	{
		std::cout << endl << "Bind.........OK" << endl << endl;
	}

	// We listen on the socket
	if (listen(listenfd, 5) < 0)
	{
		exit(1);

	}

	return 0;

}