// CheapMocap.cpp: dfinit le point d'entre pour l'application console.

//Include files
#include "stdafx.h"
#include "cv.h"
#include "highgui.h"
#include "../Common/Scene.h"
#include "../Common/Camera.h"
#include <vector>
#include <stdio.h>
#include <ctype.h>
#include <windows.h>
#include <winsock2.h>
#include <cstdio>
#include <io.h>
#include <iostream> 

extern int VERBOSE_MODE;
//Libraries
#pragma comment(lib,"ws2_32.lib")
#define BUF_SIZE 65535

//Definitions
WSADATA initialisation_win32; // Initialisation variable
int error; // Error variable
char* buffer = (char*)malloc(BUF_SIZE*sizeof(char)); // buffer containing sent and received data
SOCKET sockfd; // Socket ID
SOCKADDR_IN servaddr; // Informations bound to server
int phase=0; // Current phase

int main (int argc, char* argv[])
{
	cout << "CheapMocapClient initializating ..." << endl;

	// Creation of a scene
	Scene *scene = new Scene(1);

	// Initialisation winsock
	error=WSAStartup(MAKEWORD(2,2),&initialisation_win32);
	if (error!=0)
		cout << endl <<"Winsock error : "<< error << WSAGetLastError();
	else
		cout << endl << "WSAStartup  : OK" << endl;

	// Socket initialisation
	if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		perror("Socket error!");
		exit(1);
	}

	// Server informations
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(1219);

	bool connection=true;
	while(connection)
	{
		char ad[16];
		cout << "Server adress?" << endl;
		cin >> ad;
		servaddr.sin_addr.s_addr = inet_addr(ad);

		// Connection to the server
		if (connect(sockfd, (struct sockaddr *) &servaddr, sizeof(servaddr)) < 0)
		{
			perror("Connect error!");
		} 
		else
		{
			connection=false;
		}
	}

	cout << endl << "Connected ...socket " << sockfd << endl;

	int len; // Length of the buffer
	int id; // Client ID
	len = recv(sockfd,buffer,BUF_SIZE,0); // Waiting for ID reception

	id=atoi(buffer); // ID conversion

	cout << endl << "Connected ... Client " << id << " , socket " << sockfd << endl;
	cout << endl << "Waiting for cam init request ..." << endl;

	int c=0;

	// Main loop
	while(scene->getRunning())
	{
		len = recv(sockfd,buffer,BUF_SIZE,0); // Data reception

		// If length < 0, it means that the server has gone offline
		if (len<0)
		{
			cout << "Server offline ..." << endl;
			scene->setRunning(false);
			break;
		}

		// Processing, depending on the current phase
		switch(phase)
		{
			// Phase 0 : Calibration of the cameras
		case 0:
			// According to the defined protocol, calibration data must begin with 0x00
			if(buffer[0]==0x00)
			{
				cout << "File transfert request received!" << endl;

				scene->calibrate();


				// we forge the calibration paquet
				buffer[0]=0x02;

				// order : T, R et K

				char* charVar = (char*)(scene->getCam(0)->getTranslation());

				for (int j=0;j<12;j++)
				{
					buffer[1+j] = charVar[j];
				}

				charVar = (char*)(scene->getCam(0)->getRotation());

				for (int j=0;j<36;j++)
				{
					buffer[13+j] = charVar[j];
				}

				charVar = (char*)(scene->getCam(0)->getIntrinsec());

				for (int j=0;j<36;j++)
				{
					buffer[49+j] = charVar[j];
				}

				buffer[85]=0x00;

				send(sockfd, buffer, 86, 0); 

				phase=1;

				cout << "Calibration data sent to server!" << endl;

			}
			else
			{
				printf("Error data... : received :%s\n",buffer);
			}
			break;
			// Phase 1 : Initialisation of the cameras / initialisation of the objects
		case 1:
			// According to the defined protocol, initialisation data 1 must begin with 0x01

			if(buffer[0]==0x01)
			{
				cout << "Initialisation cam launched ..." << endl;
				scene->launch(1); // Launching the camera rendering
				phase = 2;	// Going on next phase
				cout << "Waiting for start request ..." << endl;
			}
			else
			{
				cout << "Error data... received : " << hex << buffer[0] << endl;
			}
			break;

			// Phase 2 : Capture, processing, and sending data
		case 2:
			// According to the defined protocol, capture data must begin with 0x02
			if(buffer[0]==0x02)
			{
				// If first paquet 
				if(scene->getCam(0)->getInit()==false)
				{
					Sleep(1);
				}
				else
				{

					scene->setCaptureRequest(true);
					c++;
					while(1)
					{

						Sleep(1);
						if(scene->getCaptureRequest()==false)
							break;

					}

				}

				buffer[0]=0x01; // We put data in buffer
				buffer[1]=id;
				buffer[2]=1;
				char* ptr_int;
				int obj_int=(int)(scene->getCam(0)->getPointsVector()->size());

				ptr_int=((char *)(&(obj_int)));

				// We fill the buffer with the nb of objects detected
				buffer[3]=ptr_int[0];
				buffer[4]=ptr_int[1];
				buffer[5]=ptr_int[2];
				buffer[6]=ptr_int[3];

				vector<CvPoint> vect = *(scene->getCam(0)->getPointsVector());

				char *charPoint;

				// We put the coord of the objects detected
				for(int j=0;j<(int)(scene->getCam(0)->getPointsVector()->size());j++)
				{

					charPoint=reinterpret_cast<char*>(&(vect[j]));

					for(int i=0;i<8;i++)
					{
						buffer[7+j*8+i]=charPoint[i];
					}
				}
				if(VERBOSE_MODE){
					cout << "Phase : " << buffer[0] << " , Id : " << buffer[1] << " , NbObject : " << buffer[3] << endl;
				}
				send(sockfd, buffer, 7+8*((int)(scene->getCam(0)->getPointsVector()->size())), 0); // We send data
				
				if(VERBOSE_MODE) cout << "Sending data to server ..." << endl;

				scene->getCam(0)->setInit(true);
				if (VERBOSE_MODE) cout << "Waiting for another capture request ..." << endl;

			}
			else
			{
				cout << "Error data ... received : " << hex << buffer[0] << endl;
			}
			break;
		default :
			break;

		}

	}

}

