// CheapMocap.cpp: dfinit le point d'entre pour l'application console.

// Include files
#include "stdafx.h"
#include "cv.h"
#include "highgui.h"
#include "../Common/Scene.h"
#include "../Common/Camera.h"
#include <vector>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <conio.h>
#include <sys/types.h>
#include <winsock2.h>
#include <cstdio>
#include <time.h>
#include <conio.h>
#include <QtCore>
#include <iostream>
#include "../Common/Cv32fUtils.h"
#include "../Common/Epipolar.h"
#include "../Common/trackEpipol.h"
#include "matching3D.h"

extern int VERBOSE_MODE;

using namespace std;
// Libraries
#pragma comment(lib,"ws2_32.lib")
#define BUF_SIZE 65535
// Definitions
WSADATA initialisation_win32; // Initialisation variable
int error; // Error variable
SOCKET sdServer,listenfd; // Sockets IDs

SOCKADDR_IN servAddr; // Informations bound to server

fd_set readfs; // Read
fd_set writefs; // Write

unsigned int maxdescriptor=0; // Maxdescriptor initialisation
SOCKET connfd[255]; // Socket array
int clientAlive[255]; // Client alive notificiation Array
int captureReceived[255]; // Capture received notification Array
int captureStarted[255]; // Capture phase started notification Array
int calibrOK[255];

char* data=(char*)malloc(BUF_SIZE*sizeof(char));

int clientMax=-1; // Id of the client whose id is the greatest
int nbClient=0; // Nb of clients connected

bool captureNeeded=false; // Capture needed boolean
bool fileNeeded=false;
bool calibrEnded = false;
bool captureStartedBool = false;

int initNetwork(); // Initialise the network
int calibrCam(Scene* scene, int i); // Calibrate the cams
int receiveCaptureInit(Scene* scene,int i); // Receive the first capture
int receiveCapture(Scene* scene, int i); // Receive the other captures


// OpenGL Thread, using QT
class Thread : public QThread
{
private : 
	Scene* sc;
	int argcT;
	char* argvT;

public:
	void setScene(Scene*);
	void setArgcT(int a);
	void setArgvT(char ar[]);

	Scene* getScene();
	void run();
};

void Thread::setScene(Scene* s)
{
	sc=s;
}

void Thread::setArgcT(int a)
{
	argcT=a;
}

void Thread::setArgvT(char *ar)
{
	argvT=ar;
}

Scene* Thread::getScene()
{
	return sc;
}

void Thread::run()
{
	sc->display(argcT,&argvT);
}

int main (int argc, char* argv[])
{
	if (argc>1){
		VERBOSE_MODE=1;
		cout << "Verbose mode ON" << endl;
	}else{
		VERBOSE_MODE=0;	
		cout << "Verbose mode OFF" << endl;
	}

	QCoreApplication app(argc, argv);

	Thread thread;
	Scene *scene = new Scene(0); // Creation of an empty scene

	int phase=0; // Phase initialisation
	cout << "Cheap Mocap Server !" << endl
		<< "-----------------------------------------------------------"  << endl
		<< "Usage :" << endl
		<< "[c] : send calibration file transfert request to clients"  << endl
		<< "	[i] :  send initialisation request to clients"  << endl
		<< "		[s] : send capture start request to clients"  << endl
		<< "Network Initializing ..."  << endl;

	initNetwork(); // Network initialisation

	cout << "Initialization complete ... Running ..."  << endl;


	struct timeval tv; // Timeout value
	memset(data,0,sizeof(data));
	int data_len = sizeof(data); // Length initialisation
	int client_len; // Length
	SOCKADDR_IN clientInfos; // Informations bound to client
	bool running = true; // Running boolean
	bool init = true;


	// Main loop
	while(running)
	{

		FD_ZERO(&readfs); // Read fd_set set to 0
		FD_SET(listenfd,&readfs); // Listening socket set in read fd_set

		// For each alive client, we monitor its socket
		for(int i=0;i<=clientMax;i++)
		{
			if(clientAlive[i]==0)
				continue;
			FD_SET(connfd[i],&readfs);
		}

		// Timeout reinitialisation
		tv.tv_sec = 0;
		tv.tv_usec = 500000;

		/* ------------ STATE MACHINE ---------- */
		// If keyboard activity
		if(_kbhit())
		{
			char c=_getch(); // We retrieve the key

			if(c==27)
			{
				running=false;
				break;
			}

			// Phase 0 : hitting c launch calibration
			if(phase==0 && c=='c')
			{
				phase=1; // Next phase

				// For each alive client, we send a calibration request
				for(int i=0;i<=clientMax;i++)
				{
					if(clientAlive[i]==0)
						continue;

					cout << "client " << i << " : calibration file transfert request sent" << endl;

					data[0]=0x00;
					data[1]=0x00;

					// We add the client socket in write fd_set, to send him the data
					FD_SET(connfd[i],&writefs);
					fileNeeded=true;
				}
			}


			// Phase 1 : hitting i starts initialisation
			if(phase==1 && c=='i')
			{

				data_len=2;
				phase = 2; // Next phase

				// For each alive client, we send an init cam request
				for(int i=0;i<=clientMax;i++)
				{
					if(clientAlive[i]==0)
						continue;

					cout << "client " << i << " : init cam request sent" << endl;

					// According to the defined protocol, data in phase 0 must begin with 0x01
					data[0]=0x01;
					data[1]=0x00;

					// We add the client socket in write fd_set, to send him the data
					FD_SET(connfd[i],&writefs);
				}
			}

			// Phase 2 : hitting s starts phase 3
			if(phase==2 && c=='s')
			{
				data_len=2;
				phase = 3; // Next phase

				// For each alive client, we send the first capture request
				for(int i=0;i<=clientMax;i++)
				{
					if(clientAlive[i]==0)
						continue;

					cout << "client " << i << " : init end object request sent" << endl;

					// According to the defined protocol, capture request must begin with 0x04
					data[0]=0x02;
					data[1]=0x00;

					// We add the client socket in write fd_set, to send him the data
					FD_SET(connfd[i],&writefs);
				}
			}
		}

		// Calibration
		if(phase==1 &&  !calibrEnded)
		{
			bool calibr=true;

			for(int i=0;i<=clientMax;i++)
			{
				if(clientAlive[i]==0)
					continue;

				calibr=calibr && calibrOK[i];
			}

			if(calibr)
			{
				calibrEnded=true;

				for(int i=0; i<=clientMax;i++)
				{
					if(clientAlive[i]==0)
						continue;
					if (VERBOSE_MODE){
						scene->getCam(i)->displaySettings();
					}
				}

				cout << "Computing new coordinates ..." << endl;
				scene->computePosition();

				for(int i=0; i<=clientMax;i++)
				{
					if(clientAlive[i]==0)
						continue;
					if(VERBOSE_MODE){
						scene->getCam(i)->displaySettings();
					}
				}

				// We launch the openGL thread to render the scene

				thread.setArgvT(*argv);
				thread.setArgcT(argc);
				thread.setScene(scene);
				thread.start();

				for(int i=0;i<scene->getNbCam();i++)
				{
					scene->getCam(i)->setP();
				}

			}
		}

		// Phase 3 : capture needed
		if(phase==3 && captureNeeded)
		{

		
			// If initialisation, we put the 3D Points in the scene without processing them
			if(init)
			{

				Sleep(1);
				for(int i=0;i<scene->getNbSpots();i++)
				{
					scene->addPoints3D(scene->getProj(i));
				}
				init=false;
			}
			// Else we track them
			else
			{
				trackingEpipolaire(scene);

				int nb = scene->getCam(0)->getPointsVector()->size();
				vector<CvPoint3D32f> vectTemp;

				for(int i=0;i<nb;i++)
				{
					vectTemp.push_back((scene->getProj(i)));
				}

				matching(scene->getTabPoints3D(),vectTemp);

				vectTemp.clear();

			}
			/*CvPoint3D32f la = scene->getProj(0);
			scene->addPoints3D(la);*/


			data_len=2;
			// For each alive client, we send an capture request
			for(int i=0;i<=clientMax;i++)
			{
				if(clientAlive[i]==0)
					continue;

				// We reset captureReceived Array
				captureReceived[i]=0;

				// cout << "client " << i << " : capture request sent" << endl;

				// According to the defined protocol, capture requests must begin with 0x02
				data[0]=0x02;
				data[1]=0x00;

				// We add the client socket in write fd_set, to send him the data
				FD_SET(connfd[i],&writefs);	
			}

			// We reset captureNeeded boolean
			captureNeeded=0;

		}

		/* ------------ STATE MACHINE END ---------- */

		// We monitor the activity of the sockets set in the Read and Write fd_set
		int retval = select(maxdescriptor + 1, &readfs, &writefs, NULL , &tv);

		// If 0 : no activity
		if (retval==0)
		{
			//printf("No activity .... clients : %d , phase : %d\n",nbClient,phase);

		}
		else
		{
			// Activity on the listening socket?
			if (FD_ISSET (listenfd, &readfs))
			{
				cout << "Incoming connection!" << endl;

				int newId;

				for(int j=0;j<255;j++)
				{
					if (clientAlive[j]==0)
					{
						newId=j;
						break;
					}
				}
				// We accept a new connection
				connfd[newId] = accept(listenfd, (struct sockaddr *) &clientInfos, &client_len);
				scene->addCam(newId);

				if((int)connfd[newId]<0)
				{
					perror("Connexion Error!");
					Sleep(10000);
					break;
				}

				cout << "New client " << newId << " (socket " << connfd[newId] << ") has logged in" << endl;

				// New client is alive
				clientAlive[newId]=1;

				// We are going to send his id to him
				//sprintf_s(data,,"%d",clientMax);
				sprintf_s(data,BUF_SIZE,"%d",newId);
				maxdescriptor=0;

				for(int j=0;j<255;j++)
				{
					if (clientAlive[j]==1)
					{	
						// We update the maxdescriptor
						if(connfd[j]>maxdescriptor)
							maxdescriptor=connfd[j];
					}

					if (clientAlive[j]==1)
						clientMax=j;

				}


				data_len=5;

				// We add the client socket in write fd_set, to send him the data
				FD_SET(connfd[newId],&writefs);

				// We update the counter variables

				nbClient++;

			}

			// Activity on an alive client socket?
			for(int i=0;i<=clientMax;i++)
			{
				if(clientAlive[i]==0)
					continue;

				// Writing activity ?
				if (FD_ISSET (connfd[i], &writefs))
				{
					int r=-1;
					// We send data to the client
					while(r<0)
					{
						r=send(connfd[i], data, data_len, 0);

						// printf(" %d bytes sent to socket %u / client%d\n",r,connfd[i],i);

						Sleep(1);
					}

					// We erase the socket from the writing fd_set
					FD_CLR(connfd[i],&writefs);

					continue;
				}

				// Reading activity ?
				if (FD_ISSET (connfd[i], &readfs))
				{
					// We receive data
					data_len=recv(connfd[i], data,BUF_SIZE, 0);

					// printf("%d bytes received from %u\n",data_len,connfd[i]);

					// If length < 0 it means that the client has gone offline
					if(data_len<0)
					{
						cout << "Client " << i << " disconnected" << endl;

						// We update the array
						clientAlive[i]=0;

						scene->removeCam(i);

						// We erase its socket from all the fd_sets
						FD_CLR(connfd[i],&readfs);
						FD_CLR(connfd[i],&writefs);

						// We close the connection
						shutdown(connfd[i],2);

						// We update counter
						nbClient--;
						continue;
					}
					else
					{
						// If calibration is not over
						if(!calibrEnded)
						{
							if(data[0]!=0x02)
							{
								cout << "File transfert error!" << endl;
							}
							else
							{

								calibrCam(scene,i);

							}

						}
						else
						{

							// If capture is not started
							if(!captureStartedBool)
							{

								// Reception of the first capture
								receiveCaptureInit(scene,i);

							}
							else
							{

								// Else we receive a capture
								receiveCapture(scene,i);

							}

							// New capture received
							captureReceived[i]=1;

							// We check if all the clients capture have been received
							captureNeeded=1;

							for(int j=0;j<=clientMax && captureNeeded;j++)
							{
								if(clientAlive[j]==0)
									continue;

								captureNeeded=captureNeeded&&captureReceived[j];
							}
						}
					}
				}
			}

			fileNeeded=false;

		}

		// We reset the Write fd_set
		FD_ZERO(&writefs);

	}
	delete(scene);

}

int initNetwork(){


	// We set our arrays to 0
	for(int i=0;i<255;i++)
	{
		clientAlive[i]=0;
		captureReceived[i]=0;
		captureStarted[i]=0;
		calibrOK[i]=0;
	}

	// Initialisation winsock
	error=WSAStartup(MAKEWORD(2,2),&initialisation_win32);
	if (error!=0)
		cout << endl << "Winsock initialisation error : " << error << WSAGetLastError();
	else
		cout << endl << "WSAStartup...OK";

	// Socket initialisation
	listenfd=socket(AF_INET,SOCK_STREAM,0);
	if (listenfd==INVALID_SOCKET)
		cout << endl << "Socket creation error : " << WSAGetLastError();
	else
		cout << endl << "Socket.......OK";

	// We update the maxdescriptor
	if(listenfd>maxdescriptor)
		maxdescriptor=listenfd;

	// Server informations
	servAddr.sin_family=AF_INET;
	servAddr.sin_addr.s_addr=INADDR_ANY; // Ecoute sur toutes les IP locales 
	servAddr.sin_port=htons(1219); // Ecoute sur le port 1219

	// We bind the socket
	if (bind(listenfd, (struct sockaddr *) &servAddr, sizeof(servAddr)) < 0)
	{
		perror("Bind error\n");
		exit(1); 
	}
	else
	{
		cout << endl << "Bind.........OK" << endl << endl;
	}

	// We listen on the socket
	if (listen(listenfd, 5) < 0)
	{
		exit(1);

	}

	return 0;

}

int calibrCam(Scene* scene,int i)
{

	// Calibration variables
	CvVect32f T;
	CvMatr32f R;
	CvMatr32f K;

	cout << "File received from client " << i << endl;

	char* dataTempT=data+1;
	char* dataTempR=data+13;
	char* dataTempK=data+49;

	T = (CvVect32f)dataTempT;
	R = (CvMatr32f)dataTempR;
	K = (CvMatr32f)dataTempK;

	// We update the properties of the cams
	scene->getCam(i)->setTranslation(T);
	scene->getCam(i)->setRotation(R);
	scene->getCam(i)->setIntrinsec(K);


	calibrOK[i]=true;

	return 0;
}

int receiveCaptureInit(Scene* scene, int i)
{

	// Reception of the data
	int nbObject=0;
	nbObject = *((int*)(data+3));
	char* dataTemp;

	CvPoint point;

	scene->getCam(i)->setNbPoints(nbObject);
	vector<CvPoint> *vect = (scene->getCam(i)->getPointsVector());
	vect->clear();


	// We set the actual number of white spots
	scene->setNbSpots(nbObject);

	if(nbObject!=0)
	{
		dataTemp = data+7;

	}

	CvPoint* pointsTemp = reinterpret_cast<CvPoint*>(dataTemp);
	for(int k=0;k<nbObject;k++)
	{
		point=(pointsTemp)[k];
		point.y = 480-point.y;
		vect->push_back(point);
	}

	cout << "Incoming INITIALISATION data from client " << i << " at time "<< clock() << endl;
	cout << "ID : " << i << " NbObject : " << nbObject << endl;

	if(VERBOSE_MODE){
		for(int j=0;j<nbObject;j++)
		{
			cout << "		Pt "<< j << " : " << (*(scene->getCam(i)->getPointsVector()))[j].x << " x " << (*(scene->getCam(i)->getPointsVector()))[j].y << endl;
		}
	}


	// New capture received
	captureStarted[i]=1;


	// We check if all the clients capture have been received
	captureStartedBool=1;

	for(int j=0;j<=clientMax && captureStartedBool;j++)
	{
		if(clientAlive[j]==0)
			continue;
		captureStartedBool=captureStartedBool&&captureStarted[j];

	}

	return 0;
}




int receiveCapture(Scene* scene,int i)
{
	// Reception of the data

	int nbObject=0;
	nbObject = *((int*)(data+3));
	char* dataTemp;

	CvPoint point;

	scene->getCam(i)->setNbPoints(nbObject);
	
	scene->getCam(i)->getRawPointsVector()->clear();

	if(nbObject!=0)
	{
		dataTemp = data+7;

	}

	CvPoint* pointsTemp = reinterpret_cast<CvPoint*>(dataTemp);
	for(int k=0;k<nbObject;k++)
	{
		point=(pointsTemp)[k];
		point.y = 480-point.y;
		scene->getCam(i)->getRawPointsVector()->push_back(point);

	}
	if(VERBOSE_MODE){
		cout << "Incoming data from client " << i << "at time" << clock() << endl
			<< "ID : " << i << ", NbObject : " << nbObject << endl;

		for(int j=0;j<nbObject;j++)
		{
			cout << "		Pt "<< j << " : " << (*(scene->getCam(i)->getRawPointsVector()))[j].x << " x " << (*(scene->getCam(i)->getRawPointsVector()))[j].y << endl;
		}
	}

	return 0;
}
