/*
*	Class Camera
* 
*
*
*/
#include "stdafx.h"
#include "Camera.h"
#include <math.h>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
using namespace std;
extern int VERBOSE_MODE;

/*********************************************************************
*                          Constructors                               *
*********************************************************************/

/* Empty Constructor*/
Camera::Camera(){
	translationVector = new float[3];
	rotationMatrix = new float [9];
	translationVectorWtoC = new float[3];
	rotationMatrixWtoC = new float [9];

	intrinsecMatrix = new float[9];
	P = new float[12];
	Pu = new float[4];
	Pv = new float[4];
	Pw = new float[4];
	init=false;
}

/* Constructor*/
Camera::Camera(int numCam, CvVect32f T, CvMatr32f R,CvMatr32f I)
{
	id=numCam;
	translationVector=T;
	rotationMatrix=R;
	intrinsecMatrix = I;
}

/*********************************************************************
*                          Accessors                                *
*********************************************************************/

int Camera::getW()
{
	return w;
}
int Camera::getH()
{
	return h;
}

bool Camera::getInit()
{
	return init;
}

vector<CvPoint>* Camera::getPointsVector()
{
	return &pointsVector;
}

vector<CvPoint>* Camera::getRawPointsVector()
{
	return &rawPointsVector;
}


/*return the identificator of the camera*/
int Camera::getId(){
	return id;
}

/*return the Vector of translation of the camera*/
CvVect32f Camera::getTranslation(){
	return translationVector;
}

/*return the roatation matrix of the camera*/
CvMatr32f Camera::getRotation(){
	return rotationMatrix;
}

/*return the World to cam Vector of translation of the camera*/
CvVect32f Camera::getTranslationWtoC(){
	return translationVectorWtoC;
}

/*return the World to cam rotation matrix of the camera*/
CvMatr32f Camera::getRotationWtoC(){
	return rotationMatrixWtoC;
}


/*return the intrinsec matrix of the camera*/
CvMatr32f Camera::getIntrinsec()
{
	return intrinsecMatrix;
}


/*return the First translation composant of the camera*/
CvVect32f Camera::getPu()
{
	return Pu;
}

/*return the second translation composant of the camera*/
CvVect32f Camera::getPv()
{
	return Pv;
}

/*return the third translation composant of the camera*/
CvVect32f Camera::getPw()
{
	return Pw;
}


/*displays the settings of the camera*/
void Camera::displaySettings()
{
	cout << "  -- Camera number : " << id << endl
	     << "  -- Translation Vector: " << endl
		 << translationVector[0] << " " << translationVector[1] <<" " << translationVector[2] << endl
		 << "  -- Rotation Vector: " << endl
		 << rotationMatrix[0] << " " << rotationMatrix[1] << " " << rotationMatrix[2] << endl
		 << rotationMatrix[3] << " " << rotationMatrix[4] << " " << rotationMatrix[5] << endl
		 << rotationMatrix[6] << " " << rotationMatrix[7] << " " << rotationMatrix[8] << endl
		 << "  -- Intrinsic Matrix : " << endl
		 << intrinsecMatrix[0] << " " << intrinsecMatrix[1] << " " << intrinsecMatrix[2] << endl
		 << intrinsecMatrix[3] << " " << intrinsecMatrix[4] << " " << intrinsecMatrix[5] << endl
		 << intrinsecMatrix[6] << " " << intrinsecMatrix[7] << " " << intrinsecMatrix[8] << endl;
}


int Camera::getNbPoints()
{
	return (int)(pointsVector.size());
}

CvMatr32f Camera::getP()
{
	return P;
}
/*********************************************************************
*                          Mutators                                 *
*********************************************************************/


void Camera::setNbPoints(int i)
{
	nbPoints = i;
}

/*set the identificator number of the camera*/
void Camera::setId(int noCam){
	id=noCam;
}

/*set the translation of the camera*/
void Camera::setTranslation(CvVect32f T){
	for(int i=0;i<3;i++)
	{
		translationVector[i]=T[i];
	}
}

/*set the rotation of the camera*/
void Camera::setRotation(CvMatr32f R){
	for(int i=0;i<9;i++)
	{
		rotationMatrix[i]=R[i];
	}
}

/*set the world to cam translation of the camera*/
void Camera::setTranslationWtoC(CvVect32f T){
	for(int i=0;i<3;i++)
	{
		translationVectorWtoC[i]=T[i];
	}
}

/*set the world to cam rotation of the camera*/
void Camera::setRotationWtoC(CvMatr32f R){
	for(int i=0;i<9;i++)
	{
		rotationMatrixWtoC[i]=R[i];
	}
}


/*set the intrinsec matrix of the camera*/
void Camera::setIntrinsec(CvMatr32f I)
{
	for(int i=0;i<9;i++)
	{
		intrinsecMatrix[i]=I[i];
	}

}

void Camera::setH(int a)
{
	h=a;
}
void Camera::setW(int a)
{
	w=a;
}

/*sets the settings of the camera from a file*/
void Camera::setCamera( char *filename ){

	cout << "Analysing file : "<< filename << endl;
	int i=0;
	float f[16];
	std::ifstream fichier( filename );
	if ( fichier ) 
	{
		stringstream buffer; 
		std::string ligne;
		while ( getline( fichier, ligne ) && !(fichier.eof())){
			f[i]=(float)atof(ligne.c_str());
			i++;

		}

		translationVector[0]=f[0];
		translationVector[1]=f[1];
		translationVector[2]=f[2];
		rotationMatrix[0]=f[3];
		rotationMatrix[1]=f[4];
		rotationMatrix[2]=f[5];
		rotationMatrix[3]=f[6];
		rotationMatrix[4]=f[7];
		rotationMatrix[5]=f[8];
		rotationMatrix[6]=f[9];
		rotationMatrix[7]=f[10];
		rotationMatrix[8]=f[11];
		intrinsecMatrix[0]=f[12];
		intrinsecMatrix[1]=0;
		intrinsecMatrix[2]=f[13];
		intrinsecMatrix[3]=0;
		intrinsecMatrix[4]=f[14];
		intrinsecMatrix[5]=f[15];
		intrinsecMatrix[6]=0;
		intrinsecMatrix[7]=0;
		intrinsecMatrix[8]=1;
		cout << "Analyse complete!" << endl;
		if (VERBOSE_MODE){
			displaySettings();
		}
	}
}

/*sets the projection matrix of the camera*/
void Camera::setP()
{
	P[0]=intrinsecMatrix[0]* rotationMatrix[0] + intrinsecMatrix[2]* rotationMatrix[6];
	P[1]=intrinsecMatrix[0]* rotationMatrix[1] + intrinsecMatrix[2]* rotationMatrix[7];
	P[2]=intrinsecMatrix[0]* rotationMatrix[2] + intrinsecMatrix[2]* rotationMatrix[8];
	P[3]=intrinsecMatrix[0]* translationVector[0] + intrinsecMatrix[2]* translationVector[2];
	P[4]=intrinsecMatrix[4]* rotationMatrix[3] + intrinsecMatrix[5]* rotationMatrix[6];
	P[5]=intrinsecMatrix[4]* rotationMatrix[4] + intrinsecMatrix[5]* rotationMatrix[7];
	P[6]=intrinsecMatrix[4]* rotationMatrix[5] + intrinsecMatrix[5]* rotationMatrix[8];
	P[7]=intrinsecMatrix[4]* translationVector[1] + intrinsecMatrix[5]* translationVector[2];
	P[8]=rotationMatrix[6];
	P[9]=rotationMatrix[7];
	P[10]=rotationMatrix[8];
	P[11]=translationVector[2];
	Pu[0] = P[0];
	Pu[1] = P[1];
	Pu[2] = P[2];
	Pu[3] = P[3];
	Pv[0] = P[4];
	Pv[1] = P[5];
	Pv[2] = P[6];
	Pv[3] = P[7];
	Pw[0] = P[8];
	Pw[1] = P[9];
	Pw[2] = P[10];
	Pw[3] = P[11];
}

void Camera::setPointsVector(vector<CvPoint> v)
{
	pointsVector=v;
}

void Camera::setRawPointsVector(vector<CvPoint> v)
{
	rawPointsVector=v;
}

void Camera::setInit(bool b)
{
	init=b;
}

	/*projects a 3D point on the plan 2D of the camera*/
CvPoint Camera::project2D(CvPoint3D32f point)
{
	float tab[3];
	for(int i =0 ;i<3;i++)
	{
		tab[i]= (P[i*4] * (point.x) + P[1+i*4] * (point.y) + P[2+i*4] * (point.z) + P[3+i*4]);
	}

	if(VERBOSE_MODE){
		cout << "projection : " << endl
			<< (int) (tab[0]/tab[2]) << endl
			<< (int) (tab[1]/tab[2]) << endl
			<< (int) (tab[2]) << endl;
	}

	return cvPoint((int) (tab[0]/tab[2]),(int) (tab[1]/tab[2]));
}

/**********************************************************************
 *                          Destructor                                *
 **********************************************************************/
Camera::~Camera(){}
