/*
 *	Class Camera
 * 
 *
 *
 */
#ifndef CAMERA_H
#define CAMERA_H

#include "stdafx.h"
#include "cv.h"
#include "highgui.h"
#include <stdio.h>
#include <ctype.h>
#include <vector>

using namespace std;

class Camera{
private :
	int id;

	// Cam system -> Chessboard system
	CvVect32f translationVector;
	CvMatr32f rotationMatrix;

	// Chessboard system -> Cam system
	CvVect32f translationVectorWtoC;
	CvMatr32f rotationMatrixWtoC;

	// Matrices with cam parameters
	CvMatr32f P;
	CvVect32f Pu ;
	CvVect32f Pv ;
	CvVect32f Pw ;

	CvMatr32f intrinsecMatrix;

	int nbPoints;
	int w;
	int h;

	// Vector of points, after processing
	vector<CvPoint> pointsVector;

	// Vector of points, before processing
	vector<CvPoint> rawPointsVector;

	bool init;


public :

	/*********************************************************************
	 *                          Constructors                             *
	 *********************************************************************/

	Camera();
	Camera(int, CvVect32f, CvMatr32f, CvMatr32f);

	/*********************************************************************
	 *                          Accessors                                *
	 *********************************************************************/
	

	int getW();
	int getH();
	int getNbPoints();
	bool getInit();		
	vector<CvPoint>* getPointsVector();
	vector<CvPoint>* getRawPointsVector();
	/*return the identificator of the camera*/
	int getId();
	/*return the Vector of translation of the camera*/
	CvVect32f getTranslation();
	/*return the rotation matrix of the camera*/
	CvMatr32f getRotation();

	/*return the Vector of world to cam translation of the camera*/
	CvVect32f getTranslationWtoC();
	/*return the world to cam rotation matrix of the camera*/
	CvMatr32f getRotationWtoC();

	/*return the intrinsec matrix of the camera*/
	CvMatr32f getIntrinsec();	
	/*return the First translation composant of the camera*/
	CvVect32f getPu();
	/*return the second translation composant of the camera*/
	CvVect32f getPv();
	/*return the third translation composant of the camera*/
	CvVect32f getPw();

	CvMatr32f Camera::getP();

	/*********************************************************************
	 *                          Mutators                                 *
	 *********************************************************************/


	void setH(int a);
	void setW(int a);
	void setInit(bool);
	void setPointsVector(vector<CvPoint>);
	void setRawPointsVector(vector<CvPoint>);
	void setNbPoints(int i);
	/*set the identificator number of the camera*/
	void setId(int);
	/*set the translation of the camera*/
	void setTranslation(CvVect32f);
	/*set the rotation of the camera*/
	void setRotation(CvMatr32f);
	/*set the world to cam translation of the camera*/
	void setTranslationWtoC(CvVect32f);
	/*set the world to cam rotation of the camera*/
	void setRotationWtoC(CvMatr32f);
	/*set the intrinsec matrix of the camera*/
	void setIntrinsec(CvMatr32f);
	/*sets the settings of the camera from a file*/
	void Camera::setP();
	/*sets the projection matrix of the camera*/
	void setCamera( char *filename );

	
	/*displays the settings of the camera*/
	void Camera::displaySettings();
	/*projects a 3D point on the plan 2D of the camera*/
	CvPoint project2D(CvPoint3D32f point);

	// Destructor
	~Camera();

};

#endif