#include "stdafx.h"
#include "cv.h"
#include <math.h>

#define PI 3.14159265
#define RADIANS 180.0/PI


CvVect32f multiplyVectByScaler(CvVect32f vect,float lambda,int size=3){
	CvVect32f res = new float[size];
	int i;
	for (i=0;i<size;i++){
		res[i]=lambda*((float)(vect[i]));
	}
	return res;
}

CvMatr32f multiplyMatrByScaler(CvMatr32f matr,float lambda,int size1=3,int size2=3){
	
	CvMatr32f res = new float[size1*size2];
	int i;
	for (i=0;i<size1*size2;i++){
		res[i]=lambda*((float)(matr[i]));
	}
	return res;
}

CvVect32f addVect(CvVect32f v1,CvVect32f v2,int size=3){
	CvVect32f res = new float[size];
	int i;
	for (i=0; i<size;i++){
		res[i]= (float)v1[i]+ (float)v2[i];
	}
	return res;
}

CvVect32f substractVect(CvVect32f v1,CvVect32f v2,int size=3){
	CvVect32f res = new float[size];
	int i;
	for (i=0; i<size;i++){
		res[i]= (float)v1[i]- (float)v2[i];
	}
	return res;
}

CvVect32f normalizeVect(CvVect32f vect,int size=3){
	CvVect32f res = new float[size];
	float coef=0;
	float temp = 0;
	for(int i=0;i< size;i++)
		temp = temp + pow(vect[i],2);
	coef = sqrt(temp);
	if (coef!=0) coef = pow(coef,-1);
	return multiplyVectByScaler(vect,coef,size);

}

CvVect32f crossProductVect(CvVect32f v1,CvVect32f v2,int size=3){
	CvVect32f res = new float[size];
	int i;
	for (i=0; i<size; i++){
		res[i]=v1[(i+1)%3]*v2[(i+2)%3]-v1[(i+2)%3]*v2[(i+1)%3];
	}
	return res;
}

float magnitudeVect(CvVect32f vect,int size=3){
	float temp=0;
	for(int i=0;i< size ; i++){
		temp = temp + pow(vect[i],2);
	}
	return sqrt(temp);
}





CvVect32f productMV(CvVect32f vect ,CvMatr32f matr,int size1=3, int size2=3){//matrice de taille size1*size2
	CvVect32f res = new float[size1];
	int i;
	for (i=0;i<size1;i++){
		res[i]=0;
		for (int j=0;j< size2;j++)
		{
			res[i]=res[i]+matr[size2*i+j]*vect[j];
		}
	}
	return res;
}

/*
* clamp : compute alpha modulo 360
*/
float clamp(float alpha){
	int d = (int)(floor(alpha)/360);
	float r = alpha - d*360;

	if (r>180)
		return r-360;
	return r;
}

CvVect32f extractAnglesFromMatr33(CvMatr32f mat){
	CvVect32f res = new float[3];
	float angle_x,angle_y,angle_z;
	float tx, ty;


	angle_y     = -asin(mat[2]);           /* Calcul de l'Angle Y */
	float C     =  cos(angle_y);
	angle_y     =  (float)(angle_y*RADIANS);

	if (fabs(C) > 0.005)                /* Gimbal lock ? */
	{
		tx      =  mat[8] / C;           /* Non, donc calcul de l'angle X */
		ty      = -mat[5]  / C;

		angle_x  = (float)(atan2(ty, tx) * RADIANS);

		tx      =  mat[0] / C;            /* Calcul de l'angle Z */
		ty      = -mat[1] / C;

		angle_z  = (float)(atan2(ty, tx) * RADIANS);
	}
	else                                   /* Gimbal lock  */
	{
		angle_x  = 0;                      /* Angle X  0 */

		tx      = mat[4];                 /* Calcul de l'angle Z */
		ty      = mat[3];

		angle_z  = (float)(atan2(ty, tx) * RADIANS);
	}

	res[0] = clamp(angle_x);  /* Modulo ;) */
	res[1] = clamp(angle_y);
	res[2] = clamp(angle_z);

	return res;

}

void computeSquareMatrixProduct(CvMatr32f res,CvMatr32f mat1,CvMatr32f mat2,int size=3){
	int i,j,k;

	for (j=0;j<size;j++){
		for(k=0;k<size;k++){
			res[size*j+k]=0;
			for (i=0;i<size;i++){
				res[size*j+k]+=mat1[size*j+i]*mat2[k+i*size];
			}
		}
	}

}

CvMatr32f transposeMatrix(CvMatr32f mat , int size1=3, int size2=3){
	CvMatr32f res = new float[size1*size2];
	for (int i=0;i<size1;i++){
		for (int j=0; j<size2;j++){
			res[size2* i +j ] = mat[size1*j + i];
		}
	}
	return res;
}
CvMatr32f computeCrossProductMatrix(CvVect32f vect, int size=3){
	CvMatr32f res = new float[size*size];
	res[0]=0;
	res[1]=-vect[2];
	res[2]=vect[1];
	res[3]=vect[2];
	res[4]=0;
	res[5]=-vect[0];
	res[6]=-vect[1];
	res[7]=vect[0];
	res[8]=0;
	return res;
}

CvMatr32f computeInverseMatrix(CvMatr32f mat, int size=3){
	CvMatr32f res = new float[9];

	//Compute the atrix determinant :
	float det = mat[0]*mat[4]*mat[8]+mat[1]*mat[5]*mat[6]+mat[3]*mat[7]*mat[2]-mat[6]*mat[4]*mat[2]-mat[3]*mat[1]*mat[8]-mat[7]*mat[5]*mat[0];
	if (det==0){
		for (int i = 0; i<9;i++)
			res[i]=0.0f;
		return res;
	}

	//Compute the coefficients of the comatrix :
	float c00=mat[4]*mat[8]-mat[7]*mat[5];
	float c01=-(mat[3]*mat[8]-mat[6]*mat[5]);
	float c02=(mat[3]*mat[7]-mat[6]*mat[4]);

	float c10=-(mat[1]*mat[8]-mat[7]*mat[2]);
	float c11=(mat[0]*mat[8]-mat[6]*mat[2]);
	float c12=-(mat[0]*mat[7]-mat[6]*mat[1]);

	float c20=(mat[1]*mat[5]-mat[4]*mat[2]);
	float c21=-(mat[0]*mat[5]-mat[3]*mat[2]);
	float c22=(mat[0]*mat[4]-mat[3]*mat[1]);


	// 1st line :
	res[0] = c00/det;
	res[1] = c10/det;
	res[2] = c20/det;

	// 2nd line :
	res[3] =c01/det;
	res[4] =c11/det; 
	res[5] =c21/det;

	// 3rd line :
	res[6] =c02/det;
	res[7] =c12/det;
	res[8] =c22/det;

	return res;
}
CvMatr32f transformMatrix33to44(CvMatr32f mat){
	CvMatr32f res = new float[16];
	// 1st line
	res[0]=mat[0];
	res[1]=mat[1];
	res[2]=mat[2];
	res[3]=0;

	// 2nd line
	res[4]=mat[3];
	res[5]=mat[4];
	res[6]=mat[5];
	res[7]=0;
 
	// 3rd line
	res[8]=mat[6];
	res[9]=mat[7];
	res[10]=mat[8];
	res[11]=0;

	// 4th line
	res[12]=0;
	res[13]=0;
	res[14]=0;
	res[15]=1;

	return res;
}

CvMatr32f computeTranslationMatrix(CvVect32f vect){
	CvMatr32f res = new float[16];

	// 1st line
	res[0]=0;
	res[1]=0;
	res[2]=0;
	res[3]=vect[0];

	// 2nd line
	res[4]=0;
	res[5]=0;
	res[6]=0;
	res[7]=vect[1];
 
	// 3rd line
	res[8]=0;
	res[9]=0;
	res[10]=0;
	res[11]=vect[2];

	// 4th line
	res[12]=0;
	res[13]=0;
	res[14]=0;
	res[15]=1;

	return res;
}

double *computeMatrixTranspCrossMatrix(double *mat,int size1,int size2){
	double *res =new double[size2*size2];
	for (int j=0;j<size2;j++){
		for(int k=0;k<size2;k++){
			res[size2*j+k]=0;
			for (int i=0;i<size1;i++){
				res[size2*j+k]+=mat[size2*i+j]*mat[i*size2+k];
			}
		}
	}
	return res;
}
double scalarProduct(CvVect32f vect1,CvVect32f vect2,int size = 3 , CvMatr32f mat = NULL){
	
	double res=0. ;
	int i;
	if (mat==NULL){

		for (i=0;i<size;i++){
			res+= vect1[i] * vect2[i] ;
		}
	}else{
		res = scalarProduct(vect1, productMV( vect2, mat , size, size),size );
	}
	return res ;

}

