#ifndef CV32UTILS_H
#define CV32UTILS_H
/**
 *
 * Cv32fUtils : 
 * this package regroups the tools to manipluates CvVect32f and CvMatr32f
 *
 */
#include "stdafx.h"
#include "cv.h"
#include "Scene.h"

/*
 * multiplyVectByScaler : compute the multiplication of a  CvVect32f by a scaler
 */
CvVect32f multiplyVectByScaler(CvVect32f vect,float lambda,int size=3);

CvMatr32f multiplyMatrByScaler(CvMatr32f matr,float lambda,int size1=3,int size2=3);
/*
 * addVect3 : compute the addition of 2 CvVect32f
 */
CvVect32f addVect(CvVect32f v1,CvVect32f v2,int size=3);

/*
 * substractVect : compute the substraction of 2 CvVect32f
 */
CvVect32f substractVect(CvVect32f v1,CvVect32f v2,int size=3);

/*
 * normalizeVect3 : Compute the normalization of a CvVect32f
 */
CvVect32f normalizeVect3(CvVect32f vect,int size=3);

/*
 * crossProductVect3 : compute the cross-product of 2 CvVect32f
 */
CvVect32f crossProductVect(CvVect32f v1,CvVect32f v2,int size=3);

/*
 * magnitudeVect3 : compute the magnitude of a CvVect32f
 */
float magnitudeVect3(CvVect32f vect,int size=3);

/*
 * poductMV : compute the product of a CvMatr32f by a CvVect32f
 */
CvVect32f productMV(CvVect32f vect ,CvMatr32f matr,int size1=3, int size2=3);

/*
 * extracAngesFromMatr33 : compute a CvVect32f containing the eulerians angles of a 3x3 CvMatr32f
 * condering the matrix M =	|   CE     -CF    -D   |
 *							|-BDE+AF BDF+AE   -BC  |  where A & B are cos and sin around X axis
 *							| ADE+BF -ADE+BE   AC  |		C & D ---------------------- Y axis
 *															E & F ---------------------- Z axis
 * M[2]=-D=-sin(angle_Y) then we get angle_Y using arcsin
 * then we can get C using cosinus
 * if (C!=0){
 *		M[0]/C=E -> angle_Z
 *		M[8]/C & M[5]/C give angle_X
 * }else {  D=1
 * 
 */
CvVect32f extractAnglesFromMatr33(CvMatr32f mat);


/*
 * computeSquareMatrixProduct : compute the product of 2 square matrix sizexsize
 */
void computeSquareMatrixProduct(CvMatr32f res,CvMatr32f mat1,CvMatr32f mat2,int size=3);
/*
 * transposeMatrix : compute the inversion of a matrix
 */
CvMatr32f transposeMatrix(CvMatr32f mat, int size1=3,int size2=3);

/*
 * computeCrossProductMatrix : only implemented for dimension 3
 * a crossproduct of 2 vectors can be represented as a product of a 
 * matrix by a vector for example (in 3dimenstions):
 *  |v1| |w1| |v2w3-v3w2| |0 -v3 v2| |w1|
 *  |v2|^|w2|=|v3w1-v1w3|=|v3 0 -v1|*|w2|
 *  |v3| |w3| |v1w2-v2w1| |-v2 v1 0| |w3|
 */
CvMatr32f computeCrossProductMatrix(CvVect32f mat , int size=3);

/*
 * computeInverseMatrix : compute the inverse matrix by the methode of 
 * transpose of commatrix multiplied by the inverse of the determinant.
 */
CvMatr32f computeInverseMatrix(CvMatr32f mat, int size=3);

/*
 * transformMatrix33to44 : compute a homogenous 4x4 matrix from a 3x3 matrix
 *   |a b c| |a b c 0|
 *   |d e f|=|d e f 0|
 *   |g h i| |g h i 0|
 *           |0 0 0 1|
 */
CvMatr32f transformMatrix33to44(CvMatr32f mat);

/* 
 * computeTranslationMatrix : compute a homogenous 4x4 matrix from a Vector 1x3
 */
CvMatr32f computeTranslationMatrix(CvVect32f vect);
double *computeMatrixTranspCrossMatrix(double *mat,int size1,int size2);

double scalarProduct(CvVect32f vect1,CvVect32f vect2,int size = 3 , CvMatr32f mat = NULL);

#endif