/*
* Epipolar : this class contains a vector of fondamental matrix for epipolar computing
*/
#include "stdafx.h"
#include "Epipolar.h"
#include "CV.h"
#include "Cv32fUtils.h"
#include "Camera.h"
#include "Scene.h"
#include <vector>


using namespace std;

// local variables
Scene myScene;
Camera cam1,cam2;


// Constructor & destructor
Epipolar::Epipolar(){}
Epipolar::~Epipolar(){}

// Accessors & mutators
CvMatr32f Epipolar::getEpipolarMatrix(int cam1, int cam2){
	return tabEpipolar[nbCam*cam1+cam2];
}


CvMatr32f Epipolar::getMatrixForNorm(int cam){
	return tabmatForNorm[cam];
}

vector<CvMatr32f> Epipolar::getTabEpipolar(){
	return tabEpipolar;
}

void Epipolar::setTabEpipolar(vector<CvMatr32f> tabMatr){
	tabEpipolar = tabMatr;
}
void Epipolar::addEpipolarMatrix(CvMatr32f* mat){
	tabEpipolar.push_back(*mat);
}


void Epipolar::addTab(CvMatr32f* mat){
		tabmatForNorm.push_back(*mat);
}

int Epipolar::getNbCam(){
	return nbCam;
}


//Computing functions
CvMatr32f computeFundamentalMatrix(Camera* cam1, Camera* cam2){
	CvMatr32f res = new float[9];
	CvMatr32f k1 = cam1->getIntrinsec();
	CvMatr32f k2 = cam2->getIntrinsec();
	CvMatr32f r1 = cam1->getRotationWtoC();
	CvMatr32f r2 = cam2->getRotationWtoC();
	CvVect32f t1 = cam1->getTranslationWtoC();
	CvVect32f t2 = cam2->getTranslationWtoC();
	
	CvVect32f Zero = new float[3]; // store a zero vector
	for (int i=0;i<3;i++)
		Zero[i]=0;
	
	CvMatr32f m,tmp1, tmp2, tmp3,tmp4;
	tmp4=new float[9];
	tmp2=new float[9];
	
	tmp3=new float[9];
	int i;
	// if this is the same camera, we return identity;
	if (cam1->getId()==cam2->getId()){
		for (i=0; i<9;i++){
			if(i%4==0)
				res[i]=1;
			else
				res[i]=0;
		}
	}else{
		m=computeCrossProductMatrix(substractVect(t2,t1));
		computeSquareMatrixProduct(res,k1,r1);
		tmp1=computeInverseMatrix(transposeMatrix(res));//new fait
		computeSquareMatrixProduct(tmp2,transposeMatrix(r2),computeInverseMatrix(k2));
		computeSquareMatrixProduct(tmp3,tmp1,m);
		computeSquareMatrixProduct(res,tmp3,tmp2);
		

	}
	delete [] tmp2;
	delete [] tmp3;
	return res;
}

CvMatr32f computeMatrNorm(Camera* cam1){
	CvMatr32f res = new float[9];
	CvMatr32f k1 = cam1->getIntrinsec();
	CvMatr32f r1 = cam1->getRotationWtoC();

	computeSquareMatrixProduct(res,transposeMatrix(r1),computeInverseMatrix(k1));
	return res;
}


void Epipolar::generateFundamentalMatrix(Scene sc){
	myScene = sc;
	int i,j;
	CvMatr32f* myMat =new CvMatr32f() ;
	CvMatr32f* MatNorm = new CvMatr32f();
	this->nbCam = sc.getNbCam();

	for (i=0; i<myScene.getNbCam();i++){
		for(j=0; j<myScene.getNbCam();j++){
			 *myMat = computeFundamentalMatrix(myScene.getCam(i),myScene.getCam(j));
			addEpipolarMatrix(myMat);
		}
		*MatNorm = computeMatrNorm(myScene.getCam(i));
		addTab(MatNorm);
	}
}
