#include "stdafx.h"
#include "MyGLTools.h"
#include <stdio.h>
#include <gl/glut.h>
#include <gl/gl.h>
#include <GL/glu.h> 
#include <math.h>
#include <ctype.h>
#include "Scene.h"
#include "Camera.h"
#include "cv.h"
#include "Cv32fUtils.h"
#include <iostream>

using namespace std;


#define PI 3.14598

static float angleX = 0.0 ,angleY=PI/2.0,angleZ=0,ratio;
static float x=0.0f,y=0.0f,z=500.00f;
static float lx=0.0f,ly=0.0f,lz=-20.0f;
static float oldMouseX=-1.0, oldMouseY=-1.0;
static int rightButtonPressed=0;
static Scene* myScene;
static short int opticAxisOn=0, chessboarAxisOn;

MyGLTools::MyGLTools(){}
MyGLTools::~MyGLTools(){}

/* *** SETUP OF THE SCENE *** */
/*
* buildCam : build the 3D-rendering of cam with the color coef.
*/
void buildCam(Camera* cam, float coef=1.0f) {
	glPushMatrix();

	// set color
	glColor3f(1.0f, 1.0f , 1.0f);

	// set the position
	glTranslatef((cam->getTranslationWtoC()[0])/5.0,(cam->getTranslationWtoC()[1])/5.0,(cam->getTranslationWtoC()[2])/5.0);

	// set the orientation
	glMultMatrixf(transformMatrix33to44(cam->getRotationWtoC()));

	glutSolidCone(5.0f,15.0f,10,2);

	// draw (or not) optic axis
	if (opticAxisOn>0){
		glColor3f(coef, 1.0f , 1.0f);
		glBegin(GL_LINES);
		glVertex3f(0.0f,0.0f,0.0f);
		glVertex3f(0.0f, 0.0f,  4000.0f);
		glEnd();
	}

	glPopMatrix();
}



/*
* buildPoint3D : build a sphere in the scene with the color coefficient coef
*/
void buildPoint3D(CvPoint3D32f point, float coef=1.0f){
	glPushMatrix();
	//set the color
	glColor3f(1.0f, coef ,1.0f);
	//set the position
	glTranslatef((float)(point.x)/5.0,(float)(point.y)/5.0,(float)(point.z)/5.0);

	glutSolidSphere(5.0f,24,24);
	glPopMatrix();
}

/*
* chessboardSystem : draws 3 axis corresponding to the chessboard coordinate system origin.
*/
void chessboardSystem(){

	glColor3f(1.0f, 0.0f , 1.0f);
	// Z-Axis
	glBegin(GL_LINES);
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f(0.0f, 0.0f,  40.0f);
	glEnd();

	// Y-Axis
	glBegin(GL_LINES);
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f(0.0f, 40.0f,0.0f);
	glEnd();
	// X-Axis
	glBegin(GL_LINES);
	glVertex3f(0.0f,0.0f,0.0f);
	glVertex3f(40.0f, 0.0f,0.0f);
	glEnd();

}



/*
* renderScene : procede to the render of the Scene
*/
void renderScene(void) {
	glClearColor(0.0f,0.0f,0.3f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Draw ground
	glColor3f(1.0f, 1.0f, 1.0f);
	glNormal3f(0.0f,1.0f,0.0f);
	for (int i=-25; i<26;i++){
		
			glBegin(GL_LINES);
			glVertex3f(200*(float)i, -500.0f, 5000.0f);
			glVertex3f(200*(float)i, -500.0f,  -5000.0f);
			glEnd();

			glBegin(GL_LINES);
			glVertex3f(-5000.0f, -500.0f, 200*(float)i);
			glVertex3f(5000.0f, -500.0f,  200*(float)i);
			glEnd();
	}
/*
	// Draw a cylinder 
	glPushMatrix();
	GLUquadricObj *quad = gluNewQuadric();			// Create A Pointer To The Quadric Object ( NEW )
	gluQuadricNormals(quad, GLU_SMOOTH);	// Create Smooth Normals
	//gluQuadricTexture(quadratic, GL_TRUE);		// Create Texture Coords 
	glTranslatef(60,60,60);
	gluCylinder(quad,10,10,120,24,60);
	glPopMatrix();
*/
	// Draw chessboard axis
	if (chessboarAxisOn>0){
		chessboardSystem();
	}

	// Draw the cameras
	for (int j =0;j<(int)(myScene->getNbCam());j++){
		glPushMatrix();
		buildCam(myScene->getCam(j), (float)j/((float)(myScene->getNbCam())));
		glPopMatrix();
	}

	for (int i=0;i<(int)(myScene->getTabPoints3D()->size());i++){
		if(myScene->getPoint3D(i).x!=-1 || myScene->getPoint3D(i).y!=-1 || myScene->getPoint3D(i).z!=-1)
			buildPoint3D(myScene->getPoint3D(i)/*,(float)i/((float)(myScene->getNbCam()))*/);
	}

	glutSwapBuffers();
}



/* *** MANAGEMENT OF THE POINT OF VIEW *** */
/*
* orientMeY : Compute a Y-axis rotation
*/
void orientMeY(float angY) {
	// compute the new lineOfSight
	lx = 20*sin(angY);
	lz = -20*cos(angY);
	glLoadIdentity();
	// compute the new point of vew
	gluLookAt(x, y, z, 
		x + lx,y + ly,z + lz,
		0.0f,1.0f,0.0f);
}


/*
* orientMeX : Compute a X-Axis rotation
*/
void orientMeX(float angX){
	//lz = sin(angX);
	ly = -40*cos(angX);
	glLoadIdentity();
	gluLookAt(x, y, z, 
		x + lx,y + ly,z + lz,
		0.0f,1.0f,0.0f);
}


/*
*	moveMeFlat : Compute a movement in the x,z plan
*/
void moveMeFlat(int direction) {
	x = x + direction*(lx)/200;
	y = y + direction*(ly)/200;
	z = z + direction*(lz)/200;

	glLoadIdentity();
	gluLookAt(x, y, z, 
		x + lx/20,y + ly/20,z + lz/20,
		0.0f,1.0f,0.0f);
}


/*
* strafMe : Compute a strafing movement.
*/
void strafMe(int direction){
	x = x + direction*(lz)/200;
	z = z + direction*(-lx)/200;
	glLoadIdentity();
	gluLookAt(x, y, z, 
		x + lx/20,y + ly/20,z + lz/20,
		0.0f,1.0f,0.0f);
}
/* 
* inputKey : keyboard handling
*/
void inputKey(int key, int x, int y) {

	switch (key) {
		case GLUT_KEY_UP : 
			moveMeFlat(50);break;
		case GLUT_KEY_DOWN : 
			moveMeFlat(-50);break;
		case GLUT_KEY_LEFT :
			strafMe(50);break;
		case GLUT_KEY_RIGHT :
			strafMe(-50);break;
		case GLUT_KEY_PAGE_DOWN:
			moveMeFlat(-100);break;
		case GLUT_KEY_PAGE_UP:
			moveMeFlat(100);break;
		case GLUT_KEY_F1 :
			if (opticAxisOn>0){
				opticAxisOn=0;
			}else{
				opticAxisOn=1;
			}
			break;
		case GLUT_KEY_F2 :
			if (chessboarAxisOn>0){
				chessboarAxisOn=0;
			}else{
				chessboarAxisOn=1;
			}
			break;



	}
}

/*
* mouseButtonHandler : modifies button flags when a button is pressed
*/
void rightButtonHandler(int button, int state, int mouseX, int mouseY){
	if(button==GLUT_RIGHT_BUTTON && state == GLUT_DOWN){
		rightButtonPressed=1;
	} else {
		rightButtonPressed=0;
	}
}
/* 
* mouseHandler : modifies the orientation of the point of view
*/
void mouseHandler(int mouseX, int mouseY){
	if (oldMouseX!=-1.0 && oldMouseY!=-1){
		if(rightButtonPressed!=0){

			if (mouseX!=oldMouseX){
				angleX += (float)(0.01f*(mouseX-oldMouseX)/abs(mouseX-oldMouseX));
				orientMeY(angleX);
			}
			if (mouseY!=oldMouseY){
				angleY += (float)(0.01f*(mouseY-oldMouseY)/abs(mouseY-oldMouseY));
				orientMeX(angleY);
			}
		}
	}
	oldMouseX=mouseX;
	oldMouseY=mouseY;
}



/* *** MANAGEMENT OF THE OPENGL WINDOW *** */
/*
* intScene : initialize the scene and diplay stuff 
*/
void initScene() {

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

	// Enable light
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// Create light components
	float ambientLight[] = { 0.2f, 0.2f, 0.2f, 1.0f };
	float diffuseLight[] = { 0.8f, 0.8f, 0.8, 1.0f };
	float specularLight[] = { 0.3f, 0.5f, 0.5f, 1.0f };
	float position[] = { 0.0f, 400.0f, -150.0f, 1.0f };

	// Assign created components to GL_LIGHT0
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambientLight);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specularLight);
	glLightfv(GL_LIGHT0, GL_POSITION, position);


}


/*
*	changesize : Manage resizing of the openG window
*/
void changeSize(int w, int h)
{

	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;

	ratio = 1.0f * w / h;
	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Set the viewport to be the entire window
	glViewport(0, 0, w, h);

	// Set the clipping volume
	gluPerspective(45,ratio,1,100000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(x, y, z, 
		x + lx,y + ly,z + lz,
		0.0f,1.0f,0.0f);
}


/*
* displayGL : display a openGL window with the scene.
*/
int MyGLTools::displayGL( int argc, char *argv[ ] , Scene* scene)
{
	cout << "  ### 3D Rendering ### " << endl;
	myScene=scene;
	glutInit(&argc, argv);

	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(1025,780);
	glutCreateWindow("Cheap MoCap");
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	initScene();

	glutSpecialFunc(inputKey);
	glutMouseFunc(rightButtonHandler);
	glutMotionFunc(mouseHandler);
	glutDisplayFunc(renderScene);
	glutIdleFunc(renderScene);

	glutReshapeFunc(changeSize);

	glutMainLoop();

	return 0;
}