/*
* Classe scene
*/
#if

#include "stdafx.h"
#include "Scene.h"
#include "Camera.h"
#include <iostream>
#include <time.h>
#include "MyGLTools.h"
#include "Cv32fUtils.h"
#include "math.h"
#include <QtCore>
extern int VERBOSE_MODE;
// We need a scene for the callback method
Scene* sceneTemp;
int iPic=0;

// Thresholds
int seuil=160;
int erosion=1;

using namespace std;

typedef struct queue //structure of the FIFO
{
	struct queue *next;
	int i;
	int j;
} queue_s;


/*Empty constructor*/
Scene::Scene(){}

/*n-cameras constructor*/
Scene::Scene(int nbCam)
{

	for(int i=0;i<nbCam;i++)
	{
		Camera *cam = new Camera();
		cam->setId(i);
		tabCamera.push_back(cam);
	}

	setCaptureRequest(false);
	setShotRequest(false);
	sceneTemp=this;
	running =true;
	nbSpots=0;
}

/* Destructor */
Scene::~Scene(){}

/* Return the points vector */
vector<CvPoint3D32f>* Scene::getTabPoints3D()
{
	return &tabPoints3D;
}

/* Add a point in the vector */
void Scene::addPoints3D(CvPoint3D32f pt)
{
	tabPoints3D.push_back(pt);

}

/* Return the index of the point */
CvPoint3D32f Scene::getPoint3D(int index){
	return tabPoints3D[index];
}

/* Return the index of the camera*/
Camera* Scene::getCam(int index){
	return tabCamera[index];
}

/* Return the number of cam in the scene*/
int Scene::getNbCam(){
	return (int)(tabCamera.size());
}

/* Add a cam */
void Scene::addCam()
{
	Camera *cam = new Camera();
	cam->setId((int)tabCamera.size());
	tabCamera.push_back(cam);
	if (VERBOSE_MODE){
		cout << "Camera " << cam->getId() <<" added" << endl;
	}
}

/* Add a cam with its id */
void Scene::addCam(int i)
{
	Camera *cam = new Camera();
	cam->setId(i);
	tabCamera.push_back(cam);
	if (VERBOSE_MODE){
		cout << "Camera " << cam->getId() << " added" << endl
			<< "Nb cam : " << tabCamera.size() << endl;
	}
}

/* Return the cam whose id is i */
Camera* Scene::getCamId(int i)
{

	for(int j=0;j<(int)(tabCamera.size());j++)
	{
		if(tabCamera[j]->getId()==i)
			return tabCamera[j];
	}
	return 0;
}

/* Remove a cam from the scene */
void Scene::removeCam(int i)
{


	Camera* cam;

	vector<Camera*>::iterator itVectorData;

	for (itVectorData = tabCamera.begin(); itVectorData!=tabCamera.end(); ++itVectorData) {

		cam = *itVectorData;
		if(cam->getId()==i)
		{
			break;
		}

	}

	tabCamera.erase(itVectorData);
	if(VERBOSE_MODE){
		cout << "Camera " << cam->getId() << " removed" << endl;
	}

}
/* Set the boolean running */
void Scene::setRunning(bool b)
{
	running = b;
}

/* Return the boolean runnin */
bool Scene::getRunning()
{
	return running;
}

/* Compute the parameters in the chessboard system */
void Scene::computePosition(){
	// Initialisations
	int i;
	CvVect32f T = new float[3]; // will store the translation vector
	CvVect32f Zero = new float[3]; // store a zero vector

	for (i=0;i<3;i++)
		Zero[i]=0;

	// we put all the coordinates in the chessboard sytem
	for (i=0;i<(int)tabCamera.size();i++){
		if(VERBOSE_MODE){
			cout << "Computing cam" << i << endl; 
		}

		T = substractVect(Zero,productMV(tabCamera[i]->getTranslation(),tabCamera[i]->getRotation()));
		tabCamera[i]->setRotationWtoC(transposeMatrix(tabCamera[i]->getRotation()));
		tabCamera[i]->setTranslationWtoC(T);


	}

	delete [] T;	
}

/* Set the actual number of white spots */
void Scene::setNbSpots(int a)
{
	nbSpots=a;
}

/* Return the actual number of white spots */
int Scene::getNbSpots()
{
	return nbSpots;
}

/* Return the boolean captureRequest */
bool Scene::getCaptureRequest()
{
	return captureRequest;
}

/* Set the boolean captureRequest */
void Scene::setCaptureRequest(bool cap)
{
	captureRequest=cap;
}

/* Return the boolean shotRequest */
bool Scene::getShotRequest()
{
	return shotRequest;
}

/* Set the boolean shotRequest */
void Scene::setShotRequest(bool s)
{
	shotRequest=s;
}


/* Render the scene */
void Scene::display(int argc, char *argv[ ]){
	MyGLTools *glt = new MyGLTools();
	int i;
	i=glt->displayGL(argc,argv, this);

}

/*Initialise the cams*/
void Scene::calibrate(){
	unsigned int i;
	char filename[8];
	for (i=0; i< tabCamera.size();i++){
		sprintf_s(filename,"camera%d",i);
		if(VERBOSE_MODE){
			cout << "Opening file " << filename << "..." << endl;
		}
		tabCamera[i]->setCamera(filename);
	}
	if(VERBOSE_MODE){
		cout << "--------------------------" << endl;
		//computePosition();
		for (i=0;i<tabCamera.size();i++)
			tabCamera[i]->displaySettings();
		cout << "--------------------------" << endl;
	}

}

/* Intern methods */
void callbackCapture(IplImage* image);
void callbackShot(IplImage* image);
void on_mouse( int event, int x, int y, int flags, void* param );
void identifyZone(unsigned char *data ,int width,int height,bool **marque,int i, int j,int *pos_x, int *pos_y,int *nb);
void methodeBertPat(IplImage * image,vector<CvPoint>* result,int *nb_points,int zone_min);
void Thr_Gauss_Thr(IplImage *src,IplImage *dst );
void GetBalles(IplImage *img, vector<CvPoint>* Points,int* taille);
void affichePoints(IplImage* img, vector<CvPoint> points,int taille);
void change_tab_indice(IplImage* image);

/* Initialise the cam for the capture */
int Scene::ShowCamVideoCapture(HWND hwnd, int width, int height)
{

	int numCam;
	VidFormat vidFmt={ width, height, 30.0};
	int ncams = cvcamGetCamerasCount( );
	cout << ncams << " cameras found!"<< endl; 
		while(true)
		{
			cout << "What cam do you choose?" << endl << "Available : 0 to " << ncams-1 << " :";
			cin >> numCam;
			if (numCam < ncams)
				break;
			cout << "Wrong cam number, try again" << endl;
		}

			cvcamSetProperty(numCam, CVCAM_PROP_ENABLE, CVCAMTRUE);
			cvcamSetProperty(numCam, CVCAM_PROP_CALLBACK,callbackCapture);

			cvcamSetProperty(numCam, CVCAM_PROP_WINDOW, &hwnd);  

			//Set Video Format Property
			cvcamSetProperty( numCam, CVCAM_PROP_SETFORMAT, &vidFmt);
			if( !cvcamInit() )
				return 0;


	getCam(0)->setW(width);
	getCam(0)->setH(height);

	if(VERBOSE_MODE){
		cout << "image : " << getCam(0)->getW() << " x " << getCam(0)->getH() << endl;
	}
	cvcamStart();	
	cvcamGetProperty(numCam,CVCAM_CAMERAPROPS,NULL);
	cvcamGetProperty(numCam,CVCAM_VIDEOFORMAT,NULL);
	cvDestroyWindow("cvcam window");

	cvNamedWindow("Visualisation");
	cvNamedWindow("image");
	cvCreateTrackbar("seuil","Visualisation",&seuil,255,NULL);
	cvCreateTrackbar("erosion","Visualisation",&erosion,3,NULL);

	cvSetMouseCallback("image",on_mouse,0);
	cvWaitKey(0);
	// weird behavioir if uncommented ... improvment to do : try to do something clean to escape the cvcam thread...
	//cvDestroyWindow("image");
	//cvDestroyWindow("Visualisation");
	return 1;
}

/* Initialise the cam for the shots */
int Scene::ShowCamVideoShot(HWND hwnd, int width, int height)
{

	int numCam;
	VidFormat vidFmt={ width, height, 30.0};
	int ncams = cvcamGetCamerasCount( );
	cout << ncams << " cameras found!"<< endl;
	while(1)
	{
		cout << "What cam do you choose?" << endl << "Available : 0 to " << ncams-1 << " :";

		cin >> numCam;
		if (numCam < ncams)
			break;
		cout << "Wrong cam number, try again" << endl;
	}

		cvcamSetProperty(numCam, CVCAM_PROP_ENABLE, CVCAMTRUE);
		cvcamSetProperty(numCam, CVCAM_PROP_CALLBACK,callbackShot);
		cvcamSetProperty(numCam, CVCAM_PROP_WINDOW, &hwnd);  

		//Set Video Format Property
		cvcamSetProperty( numCam, CVCAM_PROP_SETFORMAT, &vidFmt);
		if( !cvcamInit() )
			return 0;
	
	cvcamStart();	
	cvcamGetProperty(numCam,CVCAM_CAMERAPROPS,NULL);
	cvcamGetProperty(numCam,CVCAM_VIDEOFORMAT,NULL);

	return 1;
}

/* Launch the capture or the shots */
void Scene::launch(int i)
{

	cvNamedWindow("cvcam window");

	if(i==0)
	{
		if( ShowCamVideoShot((HWND)cvGetWindowHandle("cvcam window"), 640, 480) )
		{

			Sleep(1);
		}
	}
	else
	{
		if( ShowCamVideoCapture((HWND)cvGetWindowHandle("cvcam window"), 640, 480) )
		{

			Sleep(1);
		}
	}


}

/* Callback of the capture phase : this method is called every time a capture is taken */
void callbackCapture(IplImage* image)
{

	
	int i;
	int taille;

	vector<CvPoint> vect;
	IplImage* clicked = image;

	int k=	(int)(sceneTemp->getCam(0)->getPointsVector()->size());
	image->origin=1;

	// If initialisation, points are clicked points
	if(sceneTemp->getCam(0)->getInit()==false)
	{			
		clicked=image;

		// We track the clicked spots
		change_tab_indice(image);

		assert (image);
		// We draw the clicked spots
		for (i=0;i<k;i++){
			cvCircle(clicked,(*(sceneTemp->getCam(0)->getPointsVector()))[i],3, CV_RGB(255,0,0), -1, 8,0);
		}
		cvShowImage("image",clicked);

	}

	// Else points are detected points
	if(sceneTemp->getCaptureRequest() && sceneTemp->getCam(0)->getInit()==true)
	{
		vect.clear();

		// We get the coordinates of the spots
		GetBalles(image, &vect,&taille);

		/* We update the data of the scene */
		sceneTemp->getCam(0)->setPointsVector(vect);
		sceneTemp->getCam(0)->setNbPoints(taille);
		sceneTemp->setCaptureRequest(false);

		if(VERBOSE_MODE){
			cout << taille << " objects found" << cout;
		}

}

	// We exit the capture .. (doesn't work)
	if(sceneTemp->getRunning()==false)
	{
		cout << "Stopping capture..." << endl;
		cvcamStop();
		cvcamExit();
	}

	Sleep(1);

}

/* Callback of the shots phase : this method is called every time a capture is taken */
void callbackShot(IplImage* image)
{
	char fname[32];

	// If the shotRequest boolean is set as true, we save the pic in a file
	if(sceneTemp->getShotRequest())
	{
		sprintf_s(fname, "image.%002d.bmp",iPic);
		iPic++;
		cvSaveImage(fname,image);
		cout << "File saved : " << fname << endl;
		sceneTemp->setShotRequest(false);
	}

	// Exit
	if(sceneTemp->getRunning()==false)
	{
		cout << "Stopping capture..." << endl;
		cvcamStop();
		cvcamExit();
	}

	Sleep(1);
}

// On mouse method (used when we click on the spots 
void on_mouse( int event, int x, int y, int flags, void* param )
{

	CvPoint pt;

	// We get the cam points vector
	vector<CvPoint> *vect = sceneTemp->getCam(0)->getPointsVector();

	// If clicked
	if( event == CV_EVENT_LBUTTONDOWN )
	{

		// We need to modify the y coordinate
		y=sceneTemp->getCam(0)->getH() -y;

		// We add the clicked point to the vector
		pt = cvPoint(x,y);
		vect->push_back(pt);

		if(VERBOSE_MODE){
			cout << "image : " << sceneTemp->getCam(0)->getW() << " x " << sceneTemp->getCam(0)->getH() << endl
				<< "NB : " << vect->size() << ", X " << pt.x << " | Y " << pt.y << endl;
		}

	}

	// Right click for a non visible point
	if (event == CV_EVENT_RBUTTONDOWN)
	{	

		vect->push_back(cvPoint(-1,-1));

	}

}

// Spot detection algorithm
void identifyZone(unsigned char *data ,int width,int height,bool **marque,int i, int j,int *pos_x, int *pos_y,int *nb)
{	
	//the point 'pos_x' 'pos_y' est defined by the connexed white pixels (of the image data) starting by the pixel 'i','j' (that must be white)
	//'pos_x' 'pos_y' is the mean of all these white pixels.
	int i_loc,j_loc,tot_x,tot_y;


	//the white patch is seen as a graph. 
	queue_s *FIFO =(queue_s *)  malloc(sizeof(queue_s));//global FIFO
	queue_s *last = (queue_s *)  malloc(sizeof(queue_s));
	queue_s *prec ;//last element of the FIFO
	//initialisation
	FIFO->i=i;
	FIFO->j=j;
	FIFO->next=NULL;
	last=FIFO;

	tot_x= 0;
	tot_y=0;
	(*nb)=0;
	while (FIFO!=NULL)
	{
		tot_x += FIFO->i;
		tot_y += FIFO->j;
		(*nb)++;
		//we put neighbours if unmarqued
		if(i==0)//if we are on the left side of the image
			i_loc= FIFO->i;
		else
			i_loc = FIFO->i-1;
		for(;i_loc<= FIFO->i +1 && i_loc<height;i_loc++)
		{
			if(j==0 )
				j_loc = FIFO->j;
			else
				j_loc = FIFO->j-1;
			for(;j_loc<= FIFO->j+1 && j_loc<width;j_loc++) 
			{
				if(! (*marque)[i_loc*(width)+j_loc] && (data[i_loc*(width)+j_loc] == 0xff) )
				{
					last->next = (queue_s *)  malloc(sizeof(queue_s));
					last->next->i=i_loc;
					last->next->j=j_loc;
					last= last->next;
					last->next=NULL;
					(*marque)[i_loc*(width)+j_loc] = true;
				}
			}
		}
		prec = FIFO;
		FIFO = FIFO->next;
		free(prec);
	}
	*pos_x =  tot_x/(*nb);
	*pos_y =  tot_y/(*nb);

}

// Spot detection algorithm
void methodeBertPat(IplImage * image,vector<CvPoint>* result,int *nb_points,int zone_min=0) {
	/*prerequis : result est initialis, et vide! */

	int width = image->width ;
	int pos_x;
	int pos_y;
	int nb;
	int i;
	bool *marque;
	marque = (bool *)malloc(sizeof(bool)*image->height * image->width);

	for (i=0;i< image->height * image->width;i++)
	{
		marque[i] = false;
	}


	// because imageData is a signed char*
	unsigned char *data ;
	data =   reinterpret_cast<unsigned char *>(image->imageData);
	(*nb_points) = 0;
	for (i=0; i<image->height; i++) {
		for (int j=0; j<width ; j++) {
			if(!(marque[i*width+j])){
				if((data[i*width+j] == 0xff)) {//we seache for a white pixel that is not marked
					marque[i*width+j]=true;

					identifyZone(data,image->width,image->height,&marque,i,j,&pos_x,&pos_y,&nb);
					if(nb>=zone_min){
						(*nb_points)=(*nb_points)++;//<- thank to patrice, here you can laught
						(*result).push_back(cvPoint(pos_y,pos_x) );
					}
			 }
		 }
		}
	}

	delete [] marque;
}



/* identify a zone unmarqued of white pixels*/
void Thr_Gauss_Thr(IplImage *src,IplImage *dst ){
	IplImage *imgaux1;
	IplImage *imgaux2;
	int max_iters=1;
	int n = erosion - max_iters;
	int an = n > 0 ? n : -n;
	IplConvKernel* element = cvCreateStructuringElementEx( an*2+1, an*2+1, an, an, CV_SHAPE_ELLIPSE, 0 );

	imgaux1 = cvCreateImage(cvSize(src->width,src->height), src->depth, src->nChannels);
	imgaux2 = cvCreateImage(cvSize(src->width,src->height), src->depth, src->nChannels);
	//process a thresholding on the image with the threshlod 'seuil' defined by the user during the initialisation phase.
	cvThreshold(src,imgaux1,seuil,255,CV_THRESH_BINARY);
	//the erosion. The value of 'element' is determined by the user !
	cvErode(imgaux1,dst,element ,1);
	cvReleaseImage(&imgaux1);
	cvReleaseImage(&imgaux2);

}

/* Get the ball in the scene*/
void GetBalles(IplImage *img, vector<CvPoint>* Points,int* taille){


	IplImage* gray1;
	IplImage* temp;

	if (img==NULL){
		*taille=-1;
		return ;
	}

	// Images on which we are going to work
	gray1 = cvCreateImage(cvSize(img->width,img->height), img->depth,1);
	temp = cvCreateImage(cvSize(img->width,img->height), img->depth,1);

	cvCvtColor(img,temp,CV_BGR2GRAY);

	// Step 1
	// We work on greyscale image
	// We extract the balls
	gray1->origin=1;
	temp->origin=1;
	Thr_Gauss_Thr(temp,gray1);


	// Step 2
	// points detection
	cvShowImage("Visualisation",gray1);
	methodeBertPat(gray1,Points,taille,3);

	cvReleaseImage(&temp);
	cvReleaseImage(&gray1);
}

/* We display the points */
void affichePoints(IplImage* img, vector<CvPoint> points,int taille){
	int i=0;
	for(i=0;i<taille;i++){
		cvCircle(img,cvPoint(points[i].y,points[i].x),5,CV_RGB(0,255,0),-1);

	}
}

/* 2D tracking for the initialisation phase */
void change_tab_indice(IplImage* image){
	//during this tracking, the model doesn't move too much, 
	//the corresponding point in the previous time taken is simply the closest.
	unsigned int j;
	int k=-1;
	int min;
	int mincour;

	vector<CvPoint> PointP;

	vector<CvPoint> Point;

	vector<CvPoint> *Tabcam;

	Tabcam = (sceneTemp->getCam(0)->getPointsVector()) ;

	int s = (int)(Tabcam->size());

	int taille;

	GetBalles(image , &PointP ,&taille);

	for (int i=0 ;i<s ;i++){
		if ((*Tabcam)[i].x == -1 ){//if the ball is not seen
			continue;
		}
		for (j=0 ;j<PointP.size() ;j++){
			if (k== -1){
				min = pow((float)(PointP[j].x - (*Tabcam)[i].x ),2) + pow((float)(PointP[j].y - (*Tabcam)[i].y),2) ;
				k=j;
				continue;
			}
			mincour = pow((float)(PointP[j].x - (*Tabcam)[i].x ),2) + pow((float)(PointP[j].y - (*Tabcam)[i].y),2) ;
			if (mincour < min){
				mincour = min;
				k=j;
			}
		}

		if(k!=-1)
		{
			(*Tabcam)[i]=PointP[k];

			vector<CvPoint>::iterator itVectorData;
			itVectorData = PointP.begin();

			for(int q=0;q<k;q++)
			{
				itVectorData++;
			}


			PointP.erase(itVectorData);
		}
		k=-1;	
	}

	PointP.clear();
}

/* compute of the A matrix to resolve the system Ax = 0 obtained in the function getProj */
void Scene::A(int ptNb , double **resultat, int *nbPtPris)
{
	double *res = new double[4*2*getNbCam()];
	CvVect32f u,v,w,x;
	*nbPtPris = 0;
	for(int i=0;i< getNbCam();i++){
		if(((*(getCam(i)->getPointsVector()))[ptNb].x !=-1 ) || ((*(getCam(i)->getPointsVector()))[ptNb].x !=-1 ) ){

			w = multiplyVectByScaler(getCam(i)->getPw(),(*(getCam(i)->getPointsVector()))[ptNb].x,4);
			x = multiplyVectByScaler(getCam(i)->getPw(),(*(getCam(i)->getPointsVector()))[ptNb].y,4);

			u = (substractVect(getCam(i)->getPu(),w,4));
			v = (substractVect(getCam(i)->getPv(),x,4));
			for(int j=0;j<4;j++){
				res[(2**nbPtPris)*4+j] = u[j];
				res[(2**nbPtPris+1)*4+j] = v[j];
				
			}
			delete [] u;
			delete [] v;
			delete [] w;
			delete [] x;
			(*nbPtPris) = (*nbPtPris) + 1;
		}
	}

	*resultat = res;
}

/* compute the 3D point using the 2D position of each camera  */
CvPoint3D32f Scene::getProj(int ptNb)
{
	int nbCam = getNbCam();
	int nbPtTaken;
	double *B;
	double *D;
	CvMat* A1 =cvCreateMat (4,4,CV_64FC1);
	CvMat* S=cvCreateMat (4,4,CV_64FC1);
	CvMat* U=cvCreateMat (4,4,CV_64FC1);
	CvMat* V=cvCreateMat (4,4,CV_64FC1); 


	if (nbCam == 2 ){
		/* with only 2 cameras, we have a sqare matrix A*/
		A(ptNb,&B,&nbPtTaken);
	}else{
		/* if we have more than 2 cameras, the matrix A isn't square.
		* we have to make AtA to have a square one. */
		A(ptNb , &D, &nbPtTaken );
		B = computeMatrixTranspCrossMatrix(D,2*nbPtTaken,4);
	}

	/* test of the number of camera that see the point */
	if (nbPtTaken <2 ){
		if(VERBOSE_MODE){
			cout << "point number " << ptNb << "is seen by only " << nbPtTaken<< " cameras, the " << ptNb << "th point isn't displayed !"<< endl;
		}
		return cvPoint3D32f(-1,-1,-1);
	}

	/*cretaion of the CvMat of the matrix we want to analyse*/
	double C [16];
	for ( int i= 0;i< 16 ;i++){
		C[i]=B[i];
	}
	cvInitMatHeader( A1, 4, 4, CV_64FC1, C );

	/* use of the SVD method to compute the 3D point */
	cvSVD(A1,S,U,V, CV_SVD_V_T);



	/*research of the minimum singular value of the S matrix*/
	int ind_min = 0;
	double min = CV_MAT_ELEM(*S, double, 0, 0);
	for(int i = 1;i<4;i++){
		if (CV_MAT_ELEM(*S,  double, i, i) < min)
		{
			ind_min = i;
			min = CV_MAT_ELEM(*S,  double, i, i);
		}
	}

	/* take the corresponding value of V to reconstruct the 3D point*/
	float p1,p2,p3;
	p1 = (float) CV_MAT_ELEM(*V,  double, ind_min, 0)/CV_MAT_ELEM(*V,  double, ind_min, 3);
	p2 = (float) CV_MAT_ELEM(*V,  double, ind_min, 1)/CV_MAT_ELEM(*V, double, ind_min, 3);
	p3 = (float) CV_MAT_ELEM(*V,  double, ind_min, 2)/CV_MAT_ELEM(*V,  double, ind_min, 3);
	cvReleaseMat(&A1);
	cvReleaseMat(&U);
	cvReleaseMat(&V);
	cvReleaseMat(&S);

	/* output of the 3D compute value*/
	if(VERBOSE_MODE){
		cout << "x : " << p1 << endl
			<< "y : " << p2 << endl
			<<"z : " << p3 << endl;
	}

	delete [] B;
	if (nbCam != 2)
		delete [] D;
	return cvPoint3D32f(p1,p2,p3);
}