/*
* Classe scene
*
*/

#ifndef SCENE_H
#define SCENE_H


#include "stdafx.h"
#include "cv.h"
#include "highgui.h"
#include <stdio.h>
#include <ctype.h>
#include <vector>
#include "Camera.h"
#include "cvcam.h"
#include "time.h"
#include <QtCore>

using namespace std;

extern QSemaphore semaphore;

class Scene{
private:
	vector<Camera*> tabCamera; // Vector of cam
	vector<CvPoint3D32f> tabPoints3D; // Vector of 3D points

	bool captureRequest; // Capture request signal
	bool shotRequest; // Shot request signal
	bool running;
	int nbSpots; // Actual number of object in the scene

public:

	/* Empty Constructor*/
	Scene();

	/*Constructor for n cameras*/
	Scene(int);

	/*Destructor*/
	~Scene();

	/* 3D Point management */
	vector<CvPoint3D32f>* Scene::getTabPoints3D();
	void Scene::addPoints3D(CvPoint3D32f);
	CvPoint3D32f Scene::getPoint3D(int);

	/* cameras Management */ 

	Camera* Scene::getCam(int);
	int Scene::getNbCam();

	void Scene::addCam();
	void Scene::addCam(int );
	Camera* Scene::getCamId(int);
	void Scene::removeCam(int);

	/* Loop Management */
	void Scene::setRunning(bool);
	bool Scene::getRunning();

	/*Compute the parameters of position of the cameras in the masterCam reference*/
	void Scene::computePosition();

	void Scene::setNbSpots(int a);
	int Scene::getNbSpots();

	/* Network requests Management */
	bool Scene::getCaptureRequest();
	void Scene::setCaptureRequest(bool);
	bool Scene::getShotRequest();
	void Scene::setShotRequest(bool);

	/*Display a 3D render of the scene*/
	void Scene::display(int argc, char *argv[ ]);

	/*Initialise cams of the scene*/
	void Scene::calibrate();

	/* Capture methods */
	int Scene::ShowCamVideoCapture(HWND hwnd, int width, int height);
	int Scene::ShowCamVideoShot(HWND hwnd, int width, int height);

	/* Launch method */
	void Scene::launch(int i);

	/* compute of the A matrix to resolve the system Ax = 0 obtained if the function getProj */
	void Scene::A(int ptNb,double **resultat , int *nbPtPris);

	/* compute the 3D point using the 2D position of each camera */ 
	CvPoint3D32f Scene::getProj(int ptNb);

};

#endif